@@ -19,6 +19,8 @@ package v1beta1
 
 import (
 	"fmt"
+	"github.com/go-logr/logr"
+	zkApi "github.com/pravega/zookeeper-operator/api/v1beta1"
 	"strconv"
 	"strings"
 
@@ -34,7 +36,7 @@ const (
 
 	DefaultSolrReplicas = int32(3)
 	DefaultSolrRepo     = "library/solr"
-	DefaultSolrVersion  = "8.9"
+	DefaultSolrVersion  = "8.11"
 	DefaultSolrJavaMem  = "-Xms1g -Xmx2g"
 	DefaultSolrOpts     = ""
 	DefaultSolrLogLevel = "INFO"
@@ -89,6 +91,10 @@ type SolrCloudSpec struct {
 	// +optional
 	UpdateStrategy SolrUpdateStrategy `json:"updateStrategy,omitempty"`
 
+	// Define how Solr nodes should be available.
+	// +optional
+	Availability SolrAvailabilityOptions `json:"availability,omitempty"`
+
 	// +optional
 	BusyBoxImage *ContainerImage `json:"busyBoxImage,omitempty"`
 
@@ -100,6 +106,11 @@ type SolrCloudSpec struct {
 	// +optional
 	SolrOpts string `json:"solrOpts,omitempty"`
 
+	// This will add java system properties for connecting to Zookeeper.
+	// SolrZkOpts is the string interface for these optional settings
+	// +optional
+	SolrZkOpts string `json:"solrZkOpts,omitempty"`
+
 	// Set the Solr Log level, defaults to INFO
 	// +optional
 	SolrLogLevel string `json:"solrLogLevel,omitempty"`
@@ -139,7 +150,7 @@ type SolrCloudSpec struct {
 	AdditionalLibs []string `json:"additionalLibs,omitempty"`
 }
 
-func (spec *SolrCloudSpec) withDefaults() (changed bool) {
+func (spec *SolrCloudSpec) withDefaults(logger logr.Logger) (changed bool) {
 	if spec.Replicas == nil {
 		changed = true
 		r := DefaultSolrReplicas
@@ -166,7 +177,7 @@ func (spec *SolrCloudSpec) withDefaults() (changed bool) {
 		spec.SolrGCTune = DefaultSolrGCTune
 	}
 
-	changed = spec.SolrAddressability.withDefaults(spec.SolrTLS != nil) || changed
+	changed = spec.SolrAddressability.withDefaults(spec.SolrTLS != nil, logger) || changed
 
 	changed = spec.UpdateStrategy.withDefaults() || changed
 
@@ -188,19 +199,6 @@ func (spec *SolrCloudSpec) withDefaults() (changed bool) {
 	}
 	changed = spec.BusyBoxImage.withDefaults(DefaultBusyBoxImageRepo, DefaultBusyBoxImageVersion, DefaultPullPolicy) || changed
 
-	// TODO: Deprecated in v0.5.0 - remove in v0.6.0
-	if spec.StorageOptions.BackupRestoreOptions != nil {
-		spec.BackupRepositories = append(spec.BackupRepositories, SolrBackupRepository{
-			Name: LegacyBackupRepositoryName,
-			Volume: &VolumeRepository{
-				Source:    spec.StorageOptions.BackupRestoreOptions.Volume,
-				Directory: spec.StorageOptions.BackupRestoreOptions.Directory,
-			},
-		})
-		spec.StorageOptions.BackupRestoreOptions = nil
-		changed = true
-	}
-
 	return changed
 }
 
@@ -251,12 +249,6 @@ type SolrDataStorageOptions struct {
 	//
 	// +optional
 	EphemeralStorage *SolrEphemeralDataStorageOptions `json:"ephemeral,omitempty"`
-
-	// Options required for backups to be enabled for this solrCloud.
-	// Deprecated: Use a SolrBackupRepository with a VolumeRepository instead
-	// TODO: Remove in v0.6.0
-	// +optional
-	BackupRestoreOptions *SolrBackupRestoreOptions `json:"backupRestoreOptions,omitempty"`
 }
 
 func (opts *SolrDataStorageOptions) withDefaults() (changed bool) {
@@ -362,25 +354,8 @@ type SolrEphemeralDataStorageOptions struct {
 	EmptyDir *corev1.EmptyDirVolumeSource `json:"emptyDir,omitempty"`
 }
 
-// Deprecated: Use a SolrBackupRepository with a VolumeRepository instead
-type SolrBackupRestoreOptions struct {
-	// This is a volumeSource for a volume that will be mounted to all solrNodes to store backups and load restores.
-	// The data within the volume will be namespaces for this instance, so feel free to use the same volume for multiple clouds.
-	// Since the volume will be mounted to all solrNodes, it must be able to be written from multiple pods.
-	// If a PVC reference is given, the PVC must have `accessModes: - ReadWriteMany`.
-	// Other options are to use a NFS volume.
-	// Deprecated: Create an explicit 'backupRepositories' entry instead.
-	Volume corev1.VolumeSource `json:"volume"`
-
-	// Select a custom directory name to mount the backup/restore data from the given volume.
-	// If not specified, then the name of the solrcloud will be used by default.
-	// Deprecated: Create an explicit 'backupRepositories' entry instead.
-	// +optional
-	Directory string `json:"directory,omitempty"`
-}
-
-//+kubebuilder:validation:MinProperties:=2
-//+kubebuilder:validation:MaxProperties:=2
+// +kubebuilder:validation:MinProperties:=2
+// +kubebuilder:validation:MaxProperties:=2
 type SolrBackupRepository struct {
 	// A name used to identify this local storage profile.  Values should follow RFC-1123.  (See here for more details:
 	// https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#dns-label-names)
@@ -399,8 +374,6 @@ type SolrBackupRepository struct {
 	S3 *S3Repository `json:"s3,omitempty"`
 
 	// Allows specification of a "repository" for Solr to use when backing up data "locally".
-	// Repositories defined here are considered "managed" and can take advantage of special operator features, such as
-	// post-backup compression.
 	//+optional
 	Volume *VolumeRepository `json:"volume,omitempty"`
 }
@@ -409,8 +382,10 @@ type GcsRepository struct {
 	// The name of the GCS bucket that all backup data will be stored in
 	Bucket string `json:"bucket"`
 
-	// The name & key of a Kubernetes secret holding a Google cloud service account key
-	GcsCredentialSecret corev1.SecretKeySelector `json:"gcsCredentialSecret"`
+	// The name & key of a Kubernetes secret holding a Google cloud service account key.  Must be set unless deployed in
+	// GKE and making use of Google's "Workplace Identity" feature.
+	//+optional
+	GcsCredentialSecret *corev1.SecretKeySelector `json:"gcsCredentialSecret,omitempty"`
 
 	// An already-created chroot within the bucket to store data in. Defaults to the root path "/" if not specified.
 	// +optional
@@ -501,9 +476,9 @@ type SolrAddressabilityOptions struct {
 	KubeDomain string `json:"kubeDomain,omitempty"`
 }
 
-func (opts *SolrAddressabilityOptions) withDefaults(usesTLS bool) (changed bool) {
+func (opts *SolrAddressabilityOptions) withDefaults(usesTLS bool, logger logr.Logger) (changed bool) {
 	if opts.External != nil {
-		changed = opts.External.withDefaults(usesTLS)
+		changed = opts.External.withDefaults(usesTLS, logger)
 	}
 	if opts.PodPort == 0 {
 		changed = true
@@ -558,7 +533,7 @@ type ExternalAddressability struct {
 	// Provide additional domainNames that the Ingress or ExternalDNS should listen on.
 	// This option is ignored with the LoadBalancer method.
 	// +optional
-	AdditionalDomainNames []string `json:"additionalDomains,omitempty"`
+	AdditionalDomainNames []string `json:"additionalDomainNames,omitempty"`
 
 	// NodePortOverride defines the port to have all Solr node service(s) listen on and advertise itself as if advertising through an Ingress or LoadBalancer.
 	// This overrides the default usage of the podPort.
@@ -573,7 +548,7 @@ type ExternalAddressability struct {
 	// +optional
 	NodePortOverride int `json:"nodePortOverride,omitempty"`
 
-	// IngressTLSTerminationSecret defines a TLS Secret to use for TLS termination of all exposed addresses in the ingress.
+	// IngressTLSTermination tells the SolrCloud Ingress to terminate TLS on incoming connections.
 	//
 	// This is option is only available when Method=Ingress, because ExternalDNS and LoadBalancer Services do not support TLS termination.
 	// This option is also unavailable when the SolrCloud has TLS enabled via `spec.solrTLS`, in this case the Ingress cannot terminate TLS before reaching Solr.
@@ -581,10 +556,10 @@ type ExternalAddressability struct {
 	// When using this option, the UseExternalAddress option will be disabled, since Solr cannot be running in HTTP mode and making internal requests in HTTPS.
 	//
 	// +optional
-	IngressTLSTerminationSecret string `json:"ingressTLSTerminationSecret,omitempty"`
+	IngressTLSTermination *SolrIngressTLSTermination `json:"ingressTLSTermination,omitempty"`
 }
 
-// ExternalAddressability is a string enumeration type that enumerates
+// ExternalAddressabilityMethod is a string enumeration type that enumerates
 // all possible ways that a SolrCloud can be made addressable external to the kubernetes cluster.
 // +kubebuilder:validation:Enum=Ingress;ExternalDNS
 type ExternalAddressabilityMethod string
@@ -601,12 +576,13 @@ const (
 	LoadBalancer ExternalAddressabilityMethod = "LoadBalancer"
 )
 
-func (opts *ExternalAddressability) withDefaults(usesTLS bool) (changed bool) {
+func (opts *ExternalAddressability) withDefaults(usesTLS bool, logger logr.Logger) (changed bool) {
 	// You can't use an externalAddress for Solr Nodes if the Nodes are hidden externally
-	if opts.UseExternalAddress && (opts.HideNodes || opts.IngressTLSTerminationSecret != "") {
+	if opts.UseExternalAddress && (opts.HideNodes || opts.IngressTLSTermination != nil) {
 		changed = true
 		opts.UseExternalAddress = false
 	}
+
 	// If the Ingress method is used, default the nodePortOverride to 80 or 443, since that is the port that most ingress controllers listen on.
 	if !opts.HideNodes && opts.Method == Ingress && opts.NodePortOverride == 0 {
 		changed = true
@@ -625,6 +601,25 @@ func (opts *ExternalAddressability) withDefaults(usesTLS bool) (changed bool) {
 	return changed
 }
 
+// SolrIngressTLSTermination defines how a SolrCloud should have TLS Termination enabled.
+// Only one option can be provided.
+//
+// +kubebuilder:validation:MaxProperties=1
+type SolrIngressTLSTermination struct {
+
+	// UseDefaultTLSSecret determines whether the ingress should use the default TLS secret provided by the Ingress implementation.
+	//
+	// For example, using nginx: https://kubernetes.github.io/ingress-nginx/user-guide/tls/#default-ssl-certificate
+	//
+	// +optional
+	UseDefaultTLSSecret bool `json:"useDefaultTLSSecret,omitempty"`
+
+	// TLSSecret defines a TLS Secret to use for TLS termination of all exposed addresses for this SolrCloud in the Ingress.
+	//
+	// +optional
+	TLSSecret string `json:"tlsSecret,omitempty"`
+}
+
 type SolrUpdateStrategy struct {
 	// Method defines the way in which SolrClouds should be updated when the podSpec changes.
 	// +optional
@@ -649,7 +644,7 @@ type SolrUpdateStrategy struct {
 }
 
 // SolrUpdateMethod is a string enumeration type that enumerates
-// all possible ways that a SolrCloud can having rolling updates managed.
+// all possible ways that a SolrCloud can have rolling updates managed.
 // +kubebuilder:validation:Enum=Managed;StatefulSet;Manual
 type SolrUpdateMethod string
 
@@ -675,7 +670,7 @@ func (opts *SolrUpdateStrategy) withDefaults() (changed bool) {
 	return changed
 }
 
-// Spec to control the desired behavior of managed rolling update.
+// ManagedUpdateOptions control the desired behavior of managed rolling update.
 type ManagedUpdateOptions struct {
 
 	// The maximum number of pods that can be unavailable during the update.
@@ -699,6 +694,33 @@ type ManagedUpdateOptions struct {
 	MaxShardReplicasUnavailable *intstr.IntOrString `json:"maxShardReplicasUnavailable,omitempty"`
 }
 
+type SolrAvailabilityOptions struct {
+	// Define PodDisruptionBudget(s) to ensure availability of Solr
+	// +optional
+	PodDisruptionBudget SolrPodDisruptionBudgetOptions `json:"podDisruptionBudget,omitempty"`
+}
+
+type SolrPodDisruptionBudgetOptions struct {
+	// What method should be used when creating PodDisruptionBudget(s)
+	// +kubebuilder:default=true
+	Enabled *bool `json:"enabled"`
+
+	// What method should be used when creating PodDisruptionBudget(s)
+	// +kubebuilder:default=ClusterWide
+	Method SolrPodDisruptionBudgetMethod `json:"method,omitempty"`
+}
+
+// SolrPodDisruptionBudgetMethod is a string enumeration type that enumerates
+// all possible ways that a SolrCloud can have PodDisruptionBudgets managed.
+// +kubebuilder:validation:Enum=ClusterWide
+type SolrPodDisruptionBudgetMethod string
+
+const (
+	// ClusterWidePDB will result in a single cluster-wide PDB being created to ensure availability of the SolrCloud.
+	// This will not take replica/shard readiness into account.
+	ClusterWidePDB SolrPodDisruptionBudgetMethod = "ClusterWide"
+)
+
 // ZookeeperRef defines the zookeeper ensemble for solr to connect to
 // If no ConnectionString is provided, the solr-cloud controller will create and manage an internal ensemble
 type ZookeeperRef struct {
@@ -750,12 +772,20 @@ type ZookeeperSpec struct {
 	// Number of members to create up for the ZK ensemble
 	// Defaults to 3
 	// +optional
+	// +kubebuilder:validation:Minimum=1
+	// +kubebuilder:default=3
 	Replicas *int32 `json:"replicas,omitempty"`
 
 	// Image of Zookeeper to run
 	// +optional
 	Image *ContainerImage `json:"image,omitempty"`
 
+	// Labels specifies the labels to attach to all resources the operator
+	// creates for the zookeeper cluster, including StatefulSet, Pod,
+	// PersistentVolumeClaim, Service, ConfigMap, et al.
+	// +optional
+	Labels map[string]string `json:"labels,omitempty"`
+
 	// Persistence is the configuration for zookeeper persistent layer.
 	// PersistentVolumeClaimSpec and VolumeReclaimPolicy can be specified in here.
 	// At anypoint only one of Persistence or Ephemeral should be present in the manifest
@@ -785,10 +815,50 @@ type ZookeeperSpec struct {
 	// +optional
 	ReadOnlyACL *ZookeeperACL `json:"readOnlyAcl,omitempty"`
 
-	// ZooKeeper ACL to use when connecting with ZK for reading operations.
-	// This ACL should have READ permission in the given chRoot.
+	// Additional Zookeeper Configuration settings
 	// +optional
 	Config ZookeeperConfig `json:"config,omitempty"`
+
+	// Probes specifies the timeout values for the Readiness and Liveness Probes
+	// for the zookeeper pods.
+	// +optional
+	Probes *zkApi.Probes `json:"probes,omitempty"`
+
+	// Volumes defines to support customized volumes
+	// +optional
+	Volumes []corev1.Volume `json:"volumes,omitempty"`
+
+	// VolumeMounts defines to support customized volumeMounts
+	// +optional
+	VolumeMounts []corev1.VolumeMount `json:"volumeMounts,omitempty"`
+
+	// MaxUnavailableReplicas defines the
+	// MaxUnavailable Replicas in pdb.
+	// Default is 1.
+	// +optional
+	// +kubebuilder:validation:Minimum=1
+	// +kubebuilder:default=1
+	MaxUnavailableReplicas int32 `json:"maxUnavailableReplicas,omitempty"`
+
+	// Containers defines to support multi containers
+	// +optional
+	Containers []corev1.Container `json:"containers,omitempty"`
+
+	// Init containers to support initialization
+	// +optional
+	InitContainers []corev1.Container `json:"initContainers,omitempty"`
+
+	// AdminServerService defines the policy to create AdminServer Service
+	// for the zookeeper cluster.
+	AdminServerService zkApi.AdminServerServicePolicy `json:"adminServerService,omitempty"`
+
+	// ClientService defines the policy to create client Service
+	// for the zookeeper cluster.
+	ClientService zkApi.ClientServicePolicy `json:"clientService,omitempty"`
+
+	// HeadlessService defines the policy to create headless Service
+	// for the zookeeper cluster.
+	HeadlessService zkApi.HeadlessServicePolicy `json:"headlessService,omitempty"`
 }
 
 type ZKPersistence struct {
@@ -975,8 +1045,7 @@ type ZookeeperPodPolicy struct {
 	// +optional
 	Env []corev1.EnvVar `json:"env,omitempty"`
 
-	// Resources is the resource requirements for the container.
-	// This field cannot be updated once the cluster is created.
+	// Resources is the resource requirements for the Zookeeper container.
 	// +optional
 	Resources corev1.ResourceRequirements `json:"resources,omitempty"`
 
@@ -1009,23 +1078,35 @@ type ZookeeperPodPolicy struct {
 	// ImagePullSecrets is a list of references to secrets in the same namespace to use for pulling any images
 	// +optional
 	ImagePullSecrets []corev1.LocalObjectReference `json:"imagePullSecrets,omitempty"`
+
+	// TopologySpreadConstraints to apply to the pods
+	// +optional
+	TopologySpreadConstraints []corev1.TopologySpreadConstraint `json:"topologySpreadConstraints,omitempty"`
 }
 
 // SolrCloudStatus defines the observed state of SolrCloud
 type SolrCloudStatus struct {
 	// SolrNodes contain the statuses of each solr node running in this solr cloud.
+	//+listType:=map
+	//+listMapKey:=name
 	SolrNodes []SolrNodeStatus `json:"solrNodes"`
 
 	// Replicas is the number of desired replicas in the cluster
+	// +kubebuilder:validation:Minimum=0
+	// +kubebuilder:default=0
 	Replicas int32 `json:"replicas"`
 
 	// PodSelector for SolrCloud pods, required by the HPA
 	PodSelector string `json:"podSelector"`
 
 	// ReadyReplicas is the number of ready replicas in the cluster
+	// +kubebuilder:validation:Minimum=0
+	// +kubebuilder:default=0
 	ReadyReplicas int32 `json:"readyReplicas"`
 
 	// UpToDateNodes is the number of Solr Node pods that are running the latest pod spec
+	// +kubebuilder:validation:Minimum=0
+	// +kubebuilder:default=0
 	UpToDateNodes int32 `json:"upToDateNodes"`
 
 	// The version of solr that the cloud is running
@@ -1049,6 +1130,7 @@ type SolrCloudStatus struct {
 
 	// BackupRestoreReady announces whether the solrCloud has the backupRestorePVC mounted to all pods
 	// and therefore is ready for backups and restores.
+	// +optional
 	BackupRestoreReady bool `json:"backupRestoreReady"`
 
 	// BackupRepositoriesAvailable lists the backupRepositories specified in the SolrCloud and whether they are available across all Pods.
@@ -1081,6 +1163,10 @@ type SolrNodeStatus struct {
 
 	// This Solr Node pod is using the latest version of solrcloud pod spec.
 	SpecUpToDate bool `json:"specUpToDate"`
+
+	// This Solr Node pod is scheduled for deletion
+	// +optional
+	ScheduledForDeletion bool `json:"scheduledForDeletion"`
 }
 
 //+kubebuilder:object:root=true
@@ -1108,8 +1194,8 @@ type SolrCloud struct {
 }
 
 // WithDefaults set default values when not defined in the spec.
-func (sc *SolrCloud) WithDefaults() bool {
-	return sc.Spec.withDefaults()
+func (sc *SolrCloud) WithDefaults(logger logr.Logger) bool {
+	return sc.Spec.withDefaults(logger)
 }
 
 func (sc *SolrCloud) GetAllSolrPodNames() []string {
@@ -1330,11 +1416,18 @@ func (sc *SolrCloud) ExternalCommonUrl(domainName string, withPort bool) (url st
 	return url
 }
 
+func (ea *ExternalAddressability) HasIngressTLSTermination() bool {
+	if ea != nil && ea.Method == Ingress && ea.IngressTLSTermination != nil {
+		return ea.IngressTLSTermination.UseDefaultTLSSecret || ea.IngressTLSTermination.TLSSecret != ""
+	}
+	return false
+}
+
 func (sc *SolrCloud) UrlScheme(external bool) string {
 	urlScheme := "http"
 	if sc.Spec.SolrTLS != nil {
 		urlScheme = "https"
-	} else if external && sc.Spec.SolrAddressability.External != nil && sc.Spec.SolrAddressability.External.Method == Ingress && sc.Spec.SolrAddressability.External.IngressTLSTerminationSecret != "" {
+	} else if external && sc.Spec.SolrAddressability.External.HasIngressTLSTermination() {
 		urlScheme = "https"
 	}
 	return urlScheme
@@ -1353,6 +1446,16 @@ func (sc *SolrCloud) UsesPersistentStorage() bool {
 	return sc.Spec.StorageOptions.PersistentStorage != nil
 }
 
+func (sc *SolrCloud) DataVolumeName() string {
+	// Set the default name of the pvc
+	if sc.Spec.StorageOptions.PersistentStorage != nil &&
+		sc.Spec.StorageOptions.PersistentStorage.PersistentVolumeClaimTemplate.ObjectMeta.Name != "" {
+		return sc.Spec.StorageOptions.PersistentStorage.PersistentVolumeClaimTemplate.ObjectMeta.Name
+	} else {
+		return "data"
+	}
+}
+
 func (sc *SolrCloud) SharedLabels() map[string]string {
 	return sc.SharedLabelsWith(map[string]string{})
 }
