@@ -30,7 +30,7 @@ import (
 	"time"
 
 	solrv1beta1 "github.com/apache/solr-operator/api/v1beta1"
-	. "github.com/onsi/ginkgo"
+	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 )
@@ -52,6 +52,7 @@ var _ = FDescribe("SolrPrometheusExporter controller - General", func() {
 		ctx context.Context
 
 		solrPrometheusExporter *solrv1beta1.SolrPrometheusExporter
+		solrRef                *solrv1beta1.SolrCloud
 	)
 
 	BeforeEach(func() {
@@ -161,6 +162,7 @@ var _ = FDescribe("SolrPrometheusExporter controller - General", func() {
 						ServiceAccountName:            testServiceAccountName,
 						Lifecycle:                     testLifecycle,
 						TopologySpreadConstraints:     testTopologySpreadConstraints,
+						DefaultInitContainerResources: testResources2,
 					},
 					DeploymentOptions: &solrv1beta1.DeploymentOptions{
 						Annotations: testDeploymentAnnotations,
@@ -272,6 +274,9 @@ var _ = FDescribe("SolrPrometheusExporter controller - General", func() {
 			Expect(service.Spec.Ports[0].TargetPort).To(Equal(intstr.FromInt(util.SolrMetricsPort)), "Wrong target port on metrics Service")
 			Expect(service.Spec.Ports[0].Name).To(Equal(util.SolrMetricsPortName), "Wrong port name on metrics Service")
 			Expect(service.Spec.Ports[0].Port).To(Equal(int32(80)), "Wrong port number on metrics Service")
+			Expect(service.Spec.Ports[0].Protocol).To(Equal(corev1.ProtocolTCP), "Wrong protocol on metrics Service")
+			Expect(service.Spec.Ports[0].AppProtocol).ToNot(BeNil(), "AppProtocol on metrics Service should not be nil")
+			Expect(*service.Spec.Ports[0].AppProtocol).To(Equal("http"), "Wrong appProtocol on metrics Service")
 		})
 	})
 
@@ -279,7 +284,6 @@ var _ = FDescribe("SolrPrometheusExporter controller - General", func() {
 		solrName := "test-solr"
 		testZkCnxString := "host-from-solr:2181"
 		testZKChroot := "/this/path"
-		var solrRef *solrv1beta1.SolrCloud
 		BeforeEach(func() {
 			solrPrometheusExporter.Spec = solrv1beta1.SolrPrometheusExporterSpec{
 				SolrReference: solrv1beta1.SolrReference{
@@ -300,6 +304,11 @@ var _ = FDescribe("SolrPrometheusExporter controller - General", func() {
 					Namespace: "default",
 				},
 				Spec: solrv1beta1.SolrCloudSpec{
+					SolrImage: &solrv1beta1.ContainerImage{
+						Tag:             "should-be-the-same",
+						PullPolicy:      corev1.PullAlways,
+						ImagePullSecret: testImagePullSecretName2,
+					},
 					ZookeeperRef: &solrv1beta1.ZookeeperRef{
 						ConnectionInfo: &solrv1beta1.ZookeeperConnectionInfo{
 							InternalConnectionString: testZkCnxString,
@@ -331,6 +340,9 @@ var _ = FDescribe("SolrPrometheusExporter controller - General", func() {
 				}
 				g.Expect(found.Spec.Template.Spec.Containers[0].Args).To(Equal(expectedArgs), "Incorrect arguments for the SolrPrometheusExporter container")
 				g.Expect(found.Spec.Template.Spec.Containers[0].Command).To(Equal([]string{util.DefaultPrometheusExporterEntrypoint}), "Incorrect command for the SolrPrometheusExporter container")
+				g.Expect(found.Spec.Template.Spec.Containers[0].Image).To(Equal(solrv1beta1.DefaultSolrRepo+":should-be-the-same"), "Incorrect image, should be pulled from the SolrCloud")
+				g.Expect(found.Spec.Template.Spec.Containers[0].ImagePullPolicy).To(Equal(corev1.PullAlways), "Incorrect imagePullPolicy, should be pulled from the SolrCloud")
+				g.Expect(found.Spec.Template.Spec.ImagePullSecrets).To(ConsistOf(corev1.LocalObjectReference{Name: testImagePullSecretName2}), "Incorrect imagePullSecrets, should be pulled from the SolrCloud")
 
 				// Env Variable Tests
 				expectedEnvVars := map[string]string{
@@ -385,11 +397,15 @@ var _ = FDescribe("SolrPrometheusExporter controller - General", func() {
 						ValueFrom: nil,
 					},
 				}
+				for _, zkAclEnvVar := range zkAclEnvVars {
+					expectedEnvVars[zkAclEnvVar.Name] = zkAclEnvVar.Value
+				}
+				for _, extraVar := range extraVars {
+					expectedEnvVars[extraVar.Name] = extraVar.Value
+				}
 				g.Expect(foundEnv[0:5]).To(Equal(zkAclEnvVars), "ZK ACL Env Vars are not correct")
 				g.Expect(foundEnv[5:len(foundEnv)-1]).To(Equal(extraVars), "Extra Env Vars are not the same as the ones provided in podOptions")
-				// Note that this check changes the variable foundEnv, so the values are no longer valid afterwards.
-				// TODO: Make this not invalidate foundEnv
-				testMetricsPodEnvVariablesWithGomega(g, expectedEnvVars, foundEnv[len(foundEnv)-1:])
+				testPodEnvVariablesWithGomega(g, expectedEnvVars, foundEnv)
 			})
 			By("Changing the ZKConnection String of the SolrCloud")
 
@@ -414,7 +430,7 @@ var _ = FDescribe("SolrPrometheusExporter controller - General", func() {
 		})
 	})
 
-	FContext("Updating a user-provided ConfigMap", func() {
+	FContext("Updating a user-provided ConfigMap | Use explicitly defined image", func() {
 		testZkCnxString := "host-from-solr:2181"
 		testZKChroot := "/this/path"
 		withUserProvidedConfigMapName := "custom-exporter-config"
@@ -433,7 +449,32 @@ var _ = FDescribe("SolrPrometheusExporter controller - General", func() {
 						ProvidedConfigMap: withUserProvidedConfigMapName,
 					},
 				},
+				Image: &solrv1beta1.ContainerImage{
+					Repository: "test/repo",
+				},
 			}
+
+			solrRef = &solrv1beta1.SolrCloud{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      "test-solr",
+					Namespace: "default",
+				},
+				Spec: solrv1beta1.SolrCloudSpec{
+					SolrImage: &solrv1beta1.ContainerImage{
+						Tag:             "should-be-the-same",
+						PullPolicy:      corev1.PullAlways,
+						ImagePullSecret: testImagePullSecretName2,
+					},
+					ZookeeperRef: &solrv1beta1.ZookeeperRef{
+						ConnectionInfo: &solrv1beta1.ZookeeperConnectionInfo{
+							InternalConnectionString: testZkCnxString,
+							ChRoot:                   testZKChroot,
+						},
+					},
+				},
+			}
+			Expect(k8sClient.Create(ctx, solrRef)).To(Succeed(), "Creating test SolrCloud for Prometheus Exporter to connect to")
+
 		})
 		FIt("has the correct resources", func() {
 			By("testing the SolrPrometheusExporter Deployment is not created without an existing configMap")
@@ -475,6 +516,11 @@ var _ = FDescribe("SolrPrometheusExporter controller - General", func() {
 					util.PrometheusExporterConfigXmlMd5Annotation: fmt.Sprintf("%x", md5.Sum([]byte(updatedConfigXml))),
 				}
 				g.Expect(found.Spec.Template.ObjectMeta.Annotations).To(Equal(expectedAnnotations), "Incorrect pod annotations after updating configMap")
+
+				g.Expect(found.Spec.Template.Spec.Containers[0].Image).To(Equal("test/repo:"+solrv1beta1.DefaultSolrVersion), "The prometheus exporter container has the wrong image, should be using the explictly defined information, not copying from the SolrCloud")
+				g.Expect(found.Spec.Template.Spec.Containers[0].ImagePullPolicy).To(Equal(corev1.PullIfNotPresent), "The prometheus exporter container has the wrong imagePullPolicy, should not be copying from the SolrCloud")
+
+				g.Expect(found.Spec.Template.Spec.ImagePullSecrets).To(BeEmpty(), "The prometheus exporter should not have any imagePullSecrets, since it's not copying image information from the SolrCloud")
 			})
 		})
 	})
