@@ -23,6 +23,7 @@ import (
 	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
 	netv1 "k8s.io/api/networking/v1"
+	"k8s.io/apimachinery/pkg/api/resource"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/util/intstr"
 	"sort"
@@ -50,12 +51,22 @@ const (
 	LogXmlMd5Annotation              = "solr.apache.org/logXmlMd5"
 	LogXmlFile                       = "log4j2.xml"
 
+	SolrIsNotStoppedReadinessCondition       = "solr.apache.org/isNotStopped"
+	SolrReplicasNotEvictedReadinessCondition = "solr.apache.org/replicasNotEvicted"
+
 	DefaultStatefulSetPodManagementPolicy = appsv1.ParallelPodManagement
 
 	DistLibs    = "/opt/solr/dist"
 	ContribLibs = "/opt/solr/contrib/%s/lib"
 )
 
+var (
+	DefaultSolrVolumePrepInitContainerMemory = resource.NewScaledQuantity(50, 6)
+	DefaultSolrVolumePrepInitContainerCPU    = resource.NewMilliQuantity(50, resource.DecimalExponent)
+	DefaultSolrZKPrepInitContainerMemory     = resource.NewScaledQuantity(200, 6)
+	DefaultSolrZKPrepInitContainerCPU        = resource.NewMilliQuantity(400, resource.DecimalExponent)
+)
+
 // GenerateStatefulSet returns a new appsv1.StatefulSet pointer generated for the SolrCloud instance
 // object: SolrCloud instance
 // replicas: the number of replicas for the SolrCloud instance
@@ -64,7 +75,7 @@ const (
 func GenerateStatefulSet(solrCloud *solr.SolrCloud, solrCloudStatus *solr.SolrCloudStatus, hostNameIPs map[string]string, reconcileConfigInfo map[string]string, tls *TLSCerts, security *SecurityConfig) *appsv1.StatefulSet {
 	terminationGracePeriod := int64(60)
 	solrPodPort := solrCloud.Spec.SolrAddressability.PodPort
-	fsGroup := int64(DefaultSolrGroup)
+	defaultFSGroup := int64(DefaultSolrGroup)
 
 	probeScheme := corev1.URISchemeHTTP
 	if tls != nil {
@@ -72,7 +83,7 @@ func GenerateStatefulSet(solrCloud *solr.SolrCloud, solrCloudStatus *solr.SolrCl
 	}
 
 	defaultProbeTimeout := int32(1)
-	defaultHandler := corev1.Handler{
+	defaultHandler := corev1.ProbeHandler{
 		HTTPGet: &corev1.HTTPGetAction{
 			Scheme: probeScheme,
 			Path:   "/solr" + DefaultProbePath,
@@ -109,6 +120,13 @@ func GenerateStatefulSet(solrCloud *solr.SolrCloud, solrCloudStatus *solr.SolrCl
 		}
 	}
 
+	// The isNotStopped readiness gate will always be used for managedUpdates
+	podReadinessGates := []corev1.PodReadinessGate{
+		{
+			ConditionType: SolrIsNotStoppedReadinessCondition,
+		},
+	}
+
 	// Keep track of the SolrOpts that the Solr Operator needs to set
 	// These will be added to the SolrOpts given by the user.
 	allSolrOpts := []string{"-DhostPort=$(SOLR_NODE_PORT)"}
@@ -134,17 +152,14 @@ func GenerateStatefulSet(solrCloud *solr.SolrCloud, solrCloudStatus *solr.SolrCl
 		},
 	}
 
-	solrDataVolumeName := "data"
-	volumeMounts := []corev1.VolumeMount{{Name: solrDataVolumeName, MountPath: "/var/solr/data"}}
+	solrDataVolumeName := solrCloud.DataVolumeName()
 
 	var pvcs []corev1.PersistentVolumeClaim
 	if solrCloud.UsesPersistentStorage() {
 		pvc := solrCloud.Spec.StorageOptions.PersistentStorage.PersistentVolumeClaimTemplate.DeepCopy()
 
 		// Set the default name of the pvc
-		if pvc.ObjectMeta.Name == "" {
-			pvc.ObjectMeta.Name = solrDataVolumeName
-		}
+		pvc.ObjectMeta.Name = solrDataVolumeName
 
 		// Set some defaults in the PVC Spec
 		if len(pvc.Spec.AccessModes) == 0 {
@@ -192,8 +207,15 @@ func GenerateStatefulSet(solrCloud *solr.SolrCloud, solrCloudStatus *solr.SolrCl
 			ephemeralVolume.VolumeSource.EmptyDir = &corev1.EmptyDirVolumeSource{}
 		}
 		solrVolumes = append(solrVolumes, ephemeralVolume)
+
+		// Add an evictPodReadinessCondition for when deleting pods with ephemeral storage
+		podReadinessGates = append(podReadinessGates, corev1.PodReadinessGate{
+			ConditionType: SolrReplicasNotEvictedReadinessCondition,
+		})
 	}
 
+	volumeMounts := []corev1.VolumeMount{{Name: solrDataVolumeName, MountPath: "/var/solr/data"}}
+
 	// Add necessary specs for backupRepos
 	backupEnvVars := make([]corev1.EnvVar, 0)
 	for _, repo := range solrCloud.Spec.BackupRepositories {
@@ -321,9 +343,9 @@ func GenerateStatefulSet(solrCloud *solr.SolrCloud, solrCloudStatus *solr.SolrCl
 	}
 
 	// Only have a postStart command to create the chRoot, if it is not '/' (which does not need to be created)
-	var postStart *corev1.Handler
+	var postStart *corev1.LifecycleHandler
 	if hasChroot {
-		postStart = &corev1.Handler{
+		postStart = &corev1.LifecycleHandler{
 			Exec: &corev1.ExecAction{
 				Command: []string{"sh", "-c", "solr zk ls ${ZK_CHROOT} -z ${ZK_SERVER} || solr zk mkroot ${ZK_CHROOT} -z ${ZK_SERVER}"},
 			},
@@ -331,7 +353,7 @@ func GenerateStatefulSet(solrCloud *solr.SolrCloud, solrCloudStatus *solr.SolrCl
 	}
 
 	// Default preStop hook
-	preStop := &corev1.Handler{
+	preStop := &corev1.LifecycleHandler{
 		Exec: &corev1.ExecAction{
 			Command: []string{"solr", "stop", "-p", strconv.Itoa(solrPodPort)},
 		},
@@ -399,21 +421,30 @@ func GenerateStatefulSet(solrCloud *solr.SolrCloud, solrCloudStatus *solr.SolrCl
 					Protocol:      "TCP",
 				},
 			},
-			LivenessProbe: &corev1.Probe{
-				InitialDelaySeconds: 20,
+			// Wait 60 seconds for Solr to startup
+			StartupProbe: &corev1.Probe{
+				InitialDelaySeconds: 10,
 				TimeoutSeconds:      defaultProbeTimeout,
 				SuccessThreshold:    1,
-				FailureThreshold:    3,
-				PeriodSeconds:       10,
-				Handler:             defaultHandler,
-			},
-			ReadinessProbe: &corev1.Probe{
-				InitialDelaySeconds: 15,
-				TimeoutSeconds:      defaultProbeTimeout,
-				SuccessThreshold:    1,
-				FailureThreshold:    3,
+				FailureThreshold:    10,
 				PeriodSeconds:       5,
-				Handler:             defaultHandler,
+				ProbeHandler:        defaultHandler,
+			},
+			// Kill Solr if it is unavailable for any 60-second period
+			LivenessProbe: &corev1.Probe{
+				TimeoutSeconds:   defaultProbeTimeout,
+				SuccessThreshold: 1,
+				FailureThreshold: 3,
+				PeriodSeconds:    20,
+				ProbeHandler:     defaultHandler,
+			},
+			// Do not route requests to solr if it is not available for any 20-second period
+			ReadinessProbe: &corev1.Probe{
+				TimeoutSeconds:   defaultProbeTimeout,
+				SuccessThreshold: 1,
+				FailureThreshold: 2,
+				PeriodSeconds:    10,
+				ProbeHandler:     defaultHandler,
 			},
 			VolumeMounts: volumeMounts,
 			Env:          envVars,
@@ -469,12 +500,13 @@ func GenerateStatefulSet(solrCloud *solr.SolrCloud, solrCloudStatus *solr.SolrCl
 				Spec: corev1.PodSpec{
 					TerminationGracePeriodSeconds: &terminationGracePeriod,
 					SecurityContext: &corev1.PodSecurityContext{
-						FSGroup: &fsGroup,
+						FSGroup: &defaultFSGroup,
 					},
 					Volumes:        solrVolumes,
 					InitContainers: initContainers,
 					HostAliases:    hostAliases,
 					Containers:     containers,
+					ReadinessGates: podReadinessGates,
 				},
 			},
 			VolumeClaimTemplates: pvcs,
@@ -513,6 +545,9 @@ func GenerateStatefulSet(solrCloud *solr.SolrCloud, solrCloudStatus *solr.SolrCl
 
 		if customPodOptions.PodSecurityContext != nil {
 			stateful.Spec.Template.Spec.SecurityContext = customPodOptions.PodSecurityContext
+			if stateful.Spec.Template.Spec.SecurityContext.FSGroup == nil {
+				stateful.Spec.Template.Spec.SecurityContext.FSGroup = &defaultFSGroup
+			}
 		}
 
 		if customPodOptions.Lifecycle != nil {
@@ -587,28 +622,72 @@ func generateSolrSetupInitContainers(solrCloud *solr.SolrCloud, solrCloudStatus
 	}
 	setupCommands := []string{"cp /tmp/solr.xml /tmp-config/solr.xml"}
 
+	// Figure out the solrUser and solrGroup to use
+	solrUser := DefaultSolrUser
+	solrFSGroup := DefaultSolrGroup
+
+	// Only add a user to the initContainer if one isn't provided in the podSecurityContext
+	// This is so that we can check if the backupDir is writable given the default user (since no user is provided)
+	addUserToInitContainer := true
+	if solrCloud.Spec.CustomSolrKubeOptions.PodOptions != nil {
+		solrPodSecurityContext := solrCloud.Spec.CustomSolrKubeOptions.PodOptions.PodSecurityContext
+
+		if solrPodSecurityContext != nil {
+			if solrPodSecurityContext.RunAsUser != nil {
+				solrUser = int(*solrPodSecurityContext.RunAsUser)
+				addUserToInitContainer = false
+			} else if solrPodSecurityContext.RunAsNonRoot != nil && *solrPodSecurityContext.RunAsNonRoot {
+				// we can't add users to the initContainer, even if we want to, since we cannot run as root.
+				addUserToInitContainer = false
+			}
+			if solrPodSecurityContext.FSGroup != nil {
+				solrFSGroup = int(*solrPodSecurityContext.FSGroup)
+			}
+		}
+	}
+
 	// Add prep for backup-restore Repositories
 	// This entails setting the correct permissions for the directory
+	solrUserAdded := false
 	for _, repo := range solrCloud.Spec.BackupRepositories {
 		if IsRepoVolume(&repo) {
 			if _, volumeMount := RepoVolumeSourceAndMount(&repo, solrCloud.Name); volumeMount != nil {
 				volumeMounts = append(volumeMounts, *volumeMount)
 
+				if addUserToInitContainer && !solrUserAdded {
+					setupCommands = append(setupCommands, fmt.Sprintf("addgroup -g %d solr", solrFSGroup))
+					setupCommands = append(setupCommands, fmt.Sprintf("adduser -u %d -G solr -H -D solr", DefaultSolrUser))
+					// Only add users once even if there are many backup repos
+					solrUserAdded = true
+				}
+				testDirCommand := "test -w " + volumeMount.MountPath
+				if addUserToInitContainer {
+					testDirCommand = fmt.Sprintf("su solr -c '%s'", testDirCommand)
+				}
 				setupCommands = append(setupCommands, fmt.Sprintf(
-					"chown -R %d:%d %s",
-					DefaultSolrUser,
-					DefaultSolrGroup,
+					"(%s || chown -R %d:%d %s)",
+					testDirCommand,
+					solrUser,
+					solrFSGroup,
 					volumeMount.MountPath))
 			}
 		}
 	}
 
+	volumePrepResources := corev1.ResourceList{
+		corev1.ResourceCPU:    *DefaultSolrVolumePrepInitContainerCPU,
+		corev1.ResourceMemory: *DefaultSolrVolumePrepInitContainerMemory,
+	}
 	volumePrepInitContainer := corev1.Container{
 		Name:            "cp-solr-xml",
 		Image:           solrCloud.Spec.BusyBoxImage.ToImageName(),
 		ImagePullPolicy: solrCloud.Spec.BusyBoxImage.PullPolicy,
 		Command:         []string{"sh", "-c", strings.Join(setupCommands, " && ")},
 		VolumeMounts:    volumeMounts,
+		Resources: corev1.ResourceRequirements{
+			Requests: volumePrepResources,
+			Limits:   volumePrepResources,
+		},
 	}
 
 	containers = append(containers, volumePrepInitContainer)
@@ -617,6 +696,17 @@ func generateSolrSetupInitContainers(solrCloud *solr.SolrCloud, solrCloudStatus
 		containers = append(containers, zkSetupContainer)
 	}
 
+	// If the user has provided custom resources for the default init containers, use them
+	customPodOptions := solrCloud.Spec.CustomSolrKubeOptions.PodOptions
+	if nil != customPodOptions {
+		resources := customPodOptions.DefaultInitContainerResources
+		if resources.Limits != nil || resources.Requests != nil {
+			for i := range containers {
+				containers[i].Resources = resources
+			}
+		}
+	}
+
 	return containers
 }
 
@@ -709,6 +799,14 @@ func GenerateAdditionalLibXMLPart(solrModules []string, additionalLibs []string)
 	return libXml
 }
 
+func getAppProtocol(solrCloud *solr.SolrCloud) *string {
+	appProtocol := "http"
+	if solrCloud.Spec.SolrTLS != nil {
+		appProtocol = "https"
+	}
+	return &appProtocol
+}
+
 // GenerateCommonService returns a new corev1.Service pointer generated for the entire SolrCloud instance
 // solrCloud: SolrCloud instance
 func GenerateCommonService(solrCloud *solr.SolrCloud) *corev1.Service {
@@ -746,7 +844,13 @@ func GenerateCommonService(solrCloud *solr.SolrCloud) *corev1.Service {
 		},
 		Spec: corev1.ServiceSpec{
 			Ports: []corev1.ServicePort{
-				{Name: SolrClientPortName, Port: int32(solrCloud.Spec.SolrAddressability.CommonServicePort), Protocol: corev1.ProtocolTCP, TargetPort: intstr.FromString(SolrClientPortName)},
+				{
+					Name:        SolrClientPortName,
+					Port:        int32(solrCloud.Spec.SolrAddressability.CommonServicePort),
+					Protocol:    corev1.ProtocolTCP,
+					TargetPort:  intstr.FromString(SolrClientPortName),
+					AppProtocol: getAppProtocol(solrCloud),
+				},
 			},
 			Selector: selectorLabels,
 		},
@@ -792,7 +896,13 @@ func GenerateHeadlessService(solrCloud *solr.SolrCloud) *corev1.Service {
 		},
 		Spec: corev1.ServiceSpec{
 			Ports: []corev1.ServicePort{
-				{Name: SolrClientPortName, Port: int32(solrCloud.NodePort()), Protocol: corev1.ProtocolTCP, TargetPort: intstr.FromString(SolrClientPortName)},
+				{
+					Name:        SolrClientPortName,
+					Port:        int32(solrCloud.NodePort()),
+					Protocol:    corev1.ProtocolTCP,
+					TargetPort:  intstr.FromString(SolrClientPortName),
+					AppProtocol: getAppProtocol(solrCloud),
+				},
 			},
 			Selector:                 selectorLabels,
 			ClusterIP:                corev1.ClusterIPNone,
@@ -832,7 +942,13 @@ func GenerateNodeService(solrCloud *solr.SolrCloud, nodeName string) *corev1.Ser
 		Spec: corev1.ServiceSpec{
 			Selector: selectorLabels,
 			Ports: []corev1.ServicePort{
-				{Name: SolrClientPortName, Port: int32(solrCloud.NodePort()), Protocol: corev1.ProtocolTCP, TargetPort: intstr.FromString(SolrClientPortName)},
+				{
+					Name:        SolrClientPortName,
+					Port:        int32(solrCloud.NodePort()),
+					Protocol:    corev1.ProtocolTCP,
+					TargetPort:  intstr.FromString(SolrClientPortName),
+					AppProtocol: getAppProtocol(solrCloud),
+				},
 			},
 			PublishNotReadyAddresses: true,
 		},
@@ -864,11 +980,14 @@ func GenerateIngress(solrCloud *solr.SolrCloud, nodeNames []string) (ingress *ne
 		ingressTLS = append(ingressTLS, netv1.IngressTLS{SecretName: solrCloud.Spec.SolrTLS.PKCS12Secret.Name})
 	} // else if using mountedTLSDir, it's likely they'll have an auto-wired TLS solution for Ingress as well via annotations
 
-	if extOpts.IngressTLSTerminationSecret != "" {
-		ingressTLS = append(ingressTLS, netv1.IngressTLS{
-			SecretName: extOpts.IngressTLSTerminationSecret,
-			Hosts:      allHosts,
-		})
+	if extOpts.HasIngressTLSTermination() {
+		newIngressTLS := netv1.IngressTLS{
+			Hosts: allHosts,
+		}
+		if extOpts.IngressTLSTermination.TLSSecret != "" {
+			newIngressTLS.SecretName = extOpts.IngressTLSTermination.TLSSecret
+		}
+		ingressTLS = append(ingressTLS, newIngressTLS)
 	}
 	solrNodesRequireTLS := solrCloud.Spec.SolrTLS != nil
 	ingressFrontedByTLS := len(ingressTLS) > 0
@@ -1035,9 +1154,16 @@ func generateZKInteractionInitContainer(solrCloud *solr.SolrCloud, solrCloudStat
 
 	if security != nil && security.SecurityJson != "" {
 		envVars = append(envVars, corev1.EnvVar{Name: "SECURITY_JSON", ValueFrom: security.SecurityJsonSrc})
+		if solrCloud.Spec.SolrZkOpts != "" {
+			envVars = append(envVars, corev1.EnvVar{Name: "ZKCLI_JVM_FLAGS", Value: solrCloud.Spec.SolrZkOpts})
+		}
 		cmd += cmdToPutSecurityJsonInZk()
 	}
 
+	zkSetupResources := corev1.ResourceList{
+		corev1.ResourceCPU:    *DefaultSolrZKPrepInitContainerCPU,
+		corev1.ResourceMemory: *DefaultSolrZKPrepInitContainerMemory,
+	}
 	if cmd != "" {
 		return true, corev1.Container{
 			Name:                     "setup-zk",
@@ -1047,6 +1173,10 @@ func generateZKInteractionInitContainer(solrCloud *solr.SolrCloud, solrCloudStat
 			TerminationMessagePolicy: "File",
 			Command:                  []string{"sh", "-c", cmd},
 			Env:                      envVars,
+			Resources: corev1.ResourceRequirements{
+				Requests: zkSetupResources,
+				Limits:   zkSetupResources,
+			},
 		}
 	}
 
@@ -1070,16 +1200,29 @@ func createZkConnectionEnvVars(solrCloud *solr.SolrCloud, solrCloudStatus *solr.
 		},
 	}
 
+	solrOpts := make([]string, 0)
+
 	// Add ACL information, if given, through Env Vars
 	allACL, readOnlyACL := solrCloud.Spec.ZookeeperRef.GetACLs()
 	if hasACLs, aclEnvs := AddACLsToEnv(allACL, readOnlyACL); hasACLs {
 		envVars = append(envVars, aclEnvs...)
 
 		// The $SOLR_ZK_CREDS_AND_ACLS parameter does not get picked up when running solr, it must be added to the SOLR_OPTS.
-		solrOpt = "$(SOLR_ZK_CREDS_AND_ACLS)"
+		solrOpts = append(solrOpts, "$(SOLR_ZK_CREDS_AND_ACLS)")
 	}
 
-	return envVars, solrOpt, len(zkChroot) > 1
+	// Add ZK Connection System Properties to Solr
+	if solrCloud.Spec.SolrZkOpts != "" {
+		envVars = append(envVars, corev1.EnvVar{
+			Name:  "SOLR_ZK_OPTS",
+			Value: solrCloud.Spec.SolrZkOpts,
+		})
+
+		// The $SOLR_ZK_OPTS parameter does not get picked up when running solr, it must be added to the SOLR_OPTS.
+		solrOpts = append(solrOpts, "$(SOLR_ZK_OPTS)")
+	}
+
+	return envVars, strings.Join(solrOpts, " "), len(zkChroot) > 1
 }
 
 func setupVolumeMountForUserProvidedConfigMapEntry(reconcileConfigInfo map[string]string, fileKey string, solrVolumes []corev1.Volume, envVar string) (*corev1.VolumeMount, *corev1.EnvVar, *corev1.Volume) {
